{[j=20/57/1$] Pasmat Options}
{$R-}

PROGRAM MakeStandAlone;

	USES Types, QuickDraw, OSIntf, ToolIntf, IntEnv, Signal, ErrMgr, Packages, SANE;

	TYPE
		Options 			= (oNone, oResType, oResNum, oResName, oResFlags, oOutput, oProgress,
							   oHdrVersion, oHdrFlags);

		JTEntry 			= RECORD
			segOff: 			INTEGER;				{ offset from beginning of segment }
			moveOp: 			INTEGER;				{ MOVE.W #nnnn,-(SP) opcode = $3F3C }
			segNum: 			INTEGER;				{ nnnn = segment number }
			trap:				INTEGER;				{ $A9F0 = _LoadSeg }
			END;

		Code0Record 		= RECORD
			aboveA5:			LONGINT;
			belowA5:			LONGINT;
			jtLen:				LONGINT;
			jtA5Off:			LONGINT;
			jtEntries:			ARRAY [0..0] OF JTEntry;
			END;
		Code0Pointer		= ^Code0Record;
		Code0Handle 		= ^Code0Pointer;

		CodeRecord			= RECORD
			jtOffset:			INTEGER;
			numJTEntries:		INTEGER;
			END;
		CodePointer 		= ^CodeRecord;
		CodeHandle			= ^CodePointer;

		StandardHeader		= RECORD
			bra:				INTEGER;
			flags:				INTEGER;
			itsType:			ResType;
			itsID:				INTEGER;
			version:			INTEGER;
			refCon: 			LONGINT;
			END;
		StdHeaderPointer	= ^StandardHeader;
		StdHeaderHandle 	= ^StdHeaderPointer;

	CONST
		kAppParmsSize		= 32;
		kOptionStringList	= 128;
		kErrorStringList	= 129;

	VAR
		gError: 			INTEGER;
		gMyName:			IEString;
		gMyRefNum:			INTEGER;
		gHandler:			SignalHandler;
		gResType:			ResType;
		gResID: 			INTEGER;
		gResName:			IEString;
		gResFlags:			INTEGER;
		gOutputFile:		IEString;
		gInputFile: 		IEString;
		gProgress:			Boolean;
		gHdrVersion:		INTEGER;
		gHdrFlags:			INTEGER;

	PROCEDURE CallProcPtr(ProcPtr: Ptr);
		INLINE $205F,									{ MOVE.L (SP)+,A0 }
			   $4E90;									{ JSR (A0) }

	PROCEDURE ErrorExit(str: Str255; err: INTEGER);

		VAR
			s:					Str255;

		BEGIN
			UseResFile(gMyRefNum);

			IF str <> '' THEN BEGIN
				WriteLn(diagnostic, '### ', gMyName, ' - Error: ', str);
			END
			ELSE BEGIN
				WriteLn(diagnostic, '### ', gMyName, ' - Aborted!');
			END;

			IF (err <> 0) THEN BEGIN
				GetSysErrText(err, @s);
				WriteLn(diagnostic, '# ', s);
			END;
			IEExit(2);
		END;

	PROCEDURE ShowUsage;

		BEGIN
			WriteLn(diagnostic, '# Usage - ', gMyName, ' [option] <application>');
			WriteLn(diagnostic, '#    -hdrversion vers        version number to include in header');
			WriteLn(diagnostic, '#    -hdrflags flags         flags to include in header');
			WriteLn(diagnostic, '#    -o name                 output file name');
			WriteLn(diagnostic, '#    -p                      write progress information');
			WriteLn(diagnostic, '#    -resflags flags         Resource flags');
			WriteLn(diagnostic, '#    -resname name           Resource name');
			WriteLn(diagnostic, '#    -resnum n               Resource number');
			WriteLn(diagnostic, '#    -restype type           Resource type');

			IEExit(0);
		END;

	FUNCTION LookupOption(id: INTEGER; sp: IEStringPtr): Options;

		VAR
			i:					Options;
			test:				Str255;

		BEGIN
			LookupOption := oNone;
			i := oNone;

			REPEAT
				i := SUCC(i);
				GetIndString(test, id, ORD(i));
				IF (test <> '') & (EqualString(sp^, test, FALSE, FALSE)) THEN BEGIN
					LookupOption := i;
				END;
			UNTIL (test = '');
		END;

	FUNCTION GetResType(sp: IEStringPtr): ResType;

		VAR
			temp:				LONGINT;
			loop:				INTEGER;

		BEGIN
			IF Length(sp^) > 4 THEN
				ErrorExit('ResType must be 4 characters or less.', 0);

			temp := 0;
			BlockMove(@sp^[1], @temp, Length(sp^));
			GetResType := ResType(temp);
		END;

	FUNCTION GetInteger(sp: IEStringPtr): INTEGER;

		VAR
			temp:				LONGINT;

		BEGIN
			StringToNum(sp^, temp);
			IF (temp > 32767) THEN
				ErrorExit('Value entered was larger than 16-bits.', 0);
			GetInteger := temp;
		END;

	PROCEDURE ValidateCode0(code0: Code0Handle);

		BEGIN
			WITH code0^^ DO BEGIN
				IF (jtA5Off <> 32) THEN
					ErrorExit('Unknown CODE 0 format - A5 offset is not 32.', 0);

				IF (jtA5Off + jtLen <> aboveA5) THEN
					ErrorExit('Unknown CODE 0 format - aboveA5 size is not right.', 0);

				IF jtLen <> 16 THEN
					ErrorExit('Too many jump table entries (should only have 2).', 0);

				WITH jtEntries[0] DO BEGIN
					IF (segOff <> 0) THEN
						ErrorExit('Offset for first jump table entry is not 0.', 0);
					IF (segNum <> 1) THEN
						ErrorExit('Segment for first jump table entry is not 1.', 0);
				END;

				WITH jtEntries[1] DO BEGIN
					IF (segOff <> 0) THEN
						ErrorExit('Offset for second jump table entry is not 0.', 0);
					IF (segNum <> 2) THEN
						ErrorExit('Segment for second jump table entry is not 2.', 0);
				END;
			END;
		END;

	PROCEDURE ValidateCode(code1: CodeHandle; codeID: INTEGER; jtValue: INTEGER);

		BEGIN
			WITH code1^^ DO BEGIN
				IF (jtOffset <> jtValue) THEN BEGIN
					WriteLn(diagnostic, '### The jtOffset for CODE ', codeID: 0, ' should be ',
							jtValue: 0, ', but its really ', jtOffset: 0, '.');
					ErrorExit('', 0);
				END;
				IF (numJTEntries <> 1) THEN BEGIN
					WriteLn(diagnostic, '### CODE ', codeID: 0,
							' should have exactly one entry point, but it has ', numJTEntries: 0,
							'.');
					ErrorExit('', 0);
				END;
			END;
		END;

	PROCEDURE ConvertAppToStandAloneCode;

		VAR
			refNum: 			INTEGER;
			code0:				Code0Handle;
			code1:				CodeHandle;
			code2:				CodeHandle;
			sizeOfGlobals:		LONGINT;
			expandedGlobals:	Handle;
			myA5:				LONGINT;
			codeSize:			LONGINT;
			address:			CStrPtr;
			err:				OSErr;
			fndrInfo:			FInfo;
			existingResource:	Handle;

		BEGIN
			refNum := OpenResFile(gInputFile);
			IF (refNum = - 1) | (ResError = resFNotFound) THEN
				ErrorExit('Error trying to open the source file.', ResError);

			code0 := Code0Handle(Get1Resource('CODE', 0));
			IF (code0 = NIL) | (ResError <> noErr) THEN
				ErrorExit('Couldnt load CODE 0 resource.', ResError);
			HNoPurge(Handle(code0));
			ValidateCode0(code0);

			code1 := CodeHandle(Get1Resource('CODE', 1));
			IF (code1 = NIL) | (ResError <> noErr) THEN
				ErrorExit('Couldnt load CODE 1 resource.', ResError);
			HNoPurge(Handle(code1));
			ValidateCode(code1, 1, 0);

			code2 := CodeHandle(Get1Resource('CODE', 2));
			IF (code2 = NIL) | (ResError <> noErr) THEN
				ErrorExit('Couldnt load CODE 2 resource.', ResError);
			HNoPurge(Handle(code2));
			ValidateCode(code2, 2, 8);

			sizeOfGlobals := code0^^.belowA5;
			expandedGlobals := NewHandle(sizeOfGlobals);
			IF expandedGlobals = NIL THEN
				ErrorExit('Couldnt allocate memory to expand A5 data.', MemError);

			IF gProgress THEN
				WriteLn('Size of globals area:    ', sizeOfGlobals: 0);

			myA5 := SetA5(ord4(expandedGlobals^) + sizeOfGlobals);
			CallProcPtr(ProcPtr(ord4(code2^) + SizeOf(CodeRecord)));
			myA5 := SetA5(myA5);

			DetachResource(Handle(code1));

			CloseResFile(refNum);

			codeSize := GetHandleSize(Handle(code1)) - SizeOf(CodeRecord);
			SetHandleSize(Handle(code1), codeSize + sizeOfGlobals + kAppParmsSize);
			IF MemError <> noErr THEN
				ErrorExit('Couldnt expand CODE 1 handle.', MemError);

			IF gProgress THEN BEGIN
				WriteLn('Original size of CODE 1: ', codeSize: 0);
				WriteLn('Current size of CODE 1:  ', GetHandleSize(Handle(code1)): 0);
			END;

			BlockMove(Ptr(ord4(code1^) + SizeOf(CodeRecord)), Ptr(code1^), codeSize);
			BlockMove(expandedGlobals^, Ptr(ord4(code1^) + codeSize), sizeOfGlobals);
			address := CStrPtr(ord4(code1^) + codeSize + sizeOfGlobals);
			FillChar(address^, 32, CHAR(0));

			WITH StdHeaderHandle(code1)^^ DO BEGIN
				flags := gHdrFlags;
				itsType := gResType;
				itsID := gResID;
				version := gHdrVersion;
				refCon := 0;
			END;

			DisposeHandle(expandedGlobals);

			refNum := OpenResFile(gOutputFile);
			IF (refNum = - 1) | (ResError = resFNotFound) THEN BEGIN
				CreateResFile(gOutputFile);
				IF (ResError <> noErr) THEN
					ErrorExit('Error trying to create the output file.', ResError);

				err := GetFInfo(gOutputFile, 0, fndrInfo);
				IF err <> noErr THEN
					ErrorExit('Error getting finder information.', err);

				fndrInfo.fdType := 'rsrc';
				fndrInfo.fdCreator := 'RSED';
				err := SetFInfo(gOutputFile, 0, fndrInfo);
				IF err <> noErr THEN
					ErrorExit('Error setting finder information.', err);

				refNum := OpenResFile(gOutputFile);
				IF (refNum = - 1) | (ResError = resFNotFound) THEN
					ErrorExit('Error trying to open the output file.', ResError);
			END
			ELSE BEGIN
				SetResLoad(FALSE);
				existingResource := Get1Resource(gResType, gResID);
				SetResLoad(TRUE);
				IF existingResource <> NIL THEN BEGIN
					RmveResource(existingResource);
					DisposeHandle(existingResource);
				END;
			END;

			AddResource(Handle(code1), gResType, gResID, gResName);
			IF ResError <> noErr THEN
				ErrorExit('Error adding the standalone resource.', ResError);

			WriteResource(Handle(code1));
			IF ResError <> noErr THEN
				ErrorExit('Error writing the standalone resource.', ResError);

			SetResAttrs(Handle(code1), gResFlags);
			IF ResError <> noErr THEN
				ErrorExit('Error setting the resource attributes.', ResError);

			CloseResFile(refNum);
		END;

	VAR
		i:					LONGINT;
		sp: 				IEStringPtr;
		opt:				Options;

	BEGIN
		InitGraf(@thePort);
		InitErrMgr('', '', TRUE);
		gHandler := IESignal(SIGINT, SIG_IGN);			{ because AppleShare responds to Cmd-. }

		gError := 0;
		gMyName := ArgV^[0]^;
		gMyRefNum := CurResFile;

		gResType := 'PROC';
		gResID := 0;
		gResName := '';
		gResFlags := 0;
		gHdrVersion := 0;
		gHdrFlags := 0;
		gOutputFile := '';
		gInputFile := '';

		i := 1;
		WHILE i < ArgC DO BEGIN
			sp := ArgV^[i];

			IF sp^[1] = '-' THEN BEGIN
				opt := LookupOption(kOptionStringList, sp);

				CASE Options(opt) OF
					oResType: BEGIN
						i := i + 1;
						sp := ArgV^[i];
						gResType := GetResType(sp);
					END;

					oResNum: BEGIN
						i := i + 1;
						sp := ArgV^[i];
						gResID := GetInteger(sp);
					END;

					oResName: BEGIN
						i := i + 1;
						sp := ArgV^[i];
						gResName := sp^;
					END;

					oResFlags: BEGIN
						i := i + 1;
						sp := ArgV^[i];
						gResFlags := GetInteger(sp);
					END;

					oOutput: BEGIN
						i := i + 1;
						sp := ArgV^[i];
						gOutputFile := sp^;
					END;

					oProgress: BEGIN
						gProgress := TRUE;
					END;

					oHdrVersion: BEGIN
						i := i + 1;
						sp := ArgV^[i];
						gHdrVersion := GetInteger(sp);
					END;

					oHdrFlags: BEGIN
						i := i + 1;
						sp := ArgV^[i];
						gHdrFlags := GetInteger(sp);
					END;

					OTHERWISE BEGIN
						WriteLn(diagnostic, '### ', gMyName, ' - Illegal option ''', sp^, '''');
						ShowUsage;
					END
				END
			END
			ELSE
				gInputFile := sp^;

			i := i + 1;
		END;

		IF gError <> 0 THEN
			IEExit(gError);

		IF gInputFile = '' THEN
			ErrorExit('You need to specify an input file.', 0);

		IF gOutputFile = '' THEN
			gOutputFile := Concat(gInputFile, '.SA');

		IF gProgress THEN BEGIN
			WriteLn('Reading from:    ', gInputFile, '');
			WriteLn('Writing to:      ', gOutputFile, '');
			WriteLn('Resource type:   ', gResType: 0);
			WriteLn('Resource number: ', gResID: 0);
			WriteLn('Resource name:   ', gResName, '');
			WriteLn('Resource flags:  ', gResFlags: 0);
		END;

		ConvertAppToStandAloneCode;

		IEExit(gError);
	END.
